@@ -124,6 +124,13 @@ class WorkflowBaseLoader(object):
         else:
             return None
 
+    @property
+    def _gene_names(self):
+        if self.data is not None:
+            return self.data.gene_names
+        else:
+            return None
+
     def __init__(self):
         if self._file_format_settings is None:
             self._file_format_settings = dict()
@@ -275,7 +282,13 @@ class WorkflowBaseLoader(object):
         :type use_no_gold_standard: bool
         """
 
-        warnings.warn("Omitting prior network data is not recommended. Use at your own risk.")
+        if use_no_prior:
+            _msg = "Omitting prior network data is not recommended. Performance will be poor. Use at your own risk."
+            warnings.warn(_msg, UserWarning)
+
+        if use_no_gold_standard:
+            _msg = "Omitting gold standard is not recommended. Output performance metrics will be meaningless."
+            warnings.warn(_msg, UserWarning)
 
         self._set_without_warning("use_no_prior", use_no_prior)
         self._set_without_warning("use_no_gold_standard", use_no_gold_standard)
@@ -489,14 +502,14 @@ class WorkflowBaseLoader(object):
 
     def read_genes(self, file=None):
         """
-        Read tf names file into tf_names
+        Read gene names file into gene_names
         """
 
         # Load the class variable if no file is passed
         file = self.gene_names_file if file is None else file
 
         if file is not None:
-            Debug.vprint("Loading TF feature names from file {file}".format(file=file), level=1)
+            Debug.vprint("Loading Gene feature names from file {file}".format(file=file), level=1)
             # Read in a dataframe with no header or index
             loader = InferelatorDataLoader(input_dir=self.input_dir, file_format_settings=self._file_format_settings)
             genes = loader.input_dataframe(file, header=None, index_col=None)
@@ -505,46 +518,67 @@ class WorkflowBaseLoader(object):
             assert genes.shape[1] == 1
             self.gene_names = genes.values.flatten().tolist()
 
+        # Use the gene names in the data file if no restrictive list is provided
+        if self.gene_names is None and self.data is not None:
+            self.gene_names = self.data.gene_names.copy()
+
     def read_priors(self, priors_file=None, gold_standard_file=None):
         """
         Read in the priors and gold standard files
         """
+
         priors_file = priors_file if priors_file is not None else self.priors_file
         gold_standard_file = gold_standard_file if gold_standard_file is not None else self.gold_standard_file
+
         loader = InferelatorDataLoader(input_dir=self.input_dir, file_format_settings=self._file_format_settings)
 
         if priors_file is not None:
+
             Debug.vprint("Loading prior data from file {file}".format(file=priors_file), level=1)
             self.priors_data = loader.input_dataframe(priors_file)
+
+            # Print debug info & check prior for duplicate indices (which will raise errors later)
             self.loaded_file_info("Priors data", self.priors_data)
+            self._check_network_labels_unique("Priors_data", priors_file, self.priors_data)
 
         if gold_standard_file is not None:
+
             Debug.vprint("Loading gold_standard data from file {file}".format(file=gold_standard_file), level=1)
             self.gold_standard = loader.input_dataframe(gold_standard_file)
-            self.loaded_file_info("Gold standard", self.gold_standard)
 
-    def validate_data(self):
+            # Print debug info & check gold standard for duplicate indices (which will raise errors later)
+            self.loaded_file_info("Gold standard", self.gold_standard)
+            self._check_network_labels_unique("Gold standard", gold_standard_file, self.gold_standard)
+
+    def validate_data(self, check_prior=True, check_gold_standard=True):
         """
         Make sure that the data that's loaded is acceptable
         """
 
-        # Create a null prior if the flag is set
-        if self.use_no_prior and self.priors_data is not None:
-            warnings.warn("The use_no_prior flag will be ignored because prior data exists")
-        elif self.use_no_prior:
-            Debug.vprint("A null prior is has been created", level=0)
-            self.priors_data = self._create_null_prior(self.data.gene_names, self.tf_names)
+        if check_prior:
+            # Create a null prior if the flag is set
+            if self.use_no_prior and self.priors_data is not None:
+                warnings.warn("The use_no_prior flag will be ignored because prior data exists")
+            elif self.use_no_prior:
+                Debug.vprint("A null prior is has been created", level=0)
+                self.priors_data = self._create_null_prior(self._gene_names, self.tf_names)
 
-        # Create a null gold standard if the flag is set
-        if self.use_no_gold_standard and self.gold_standard is not None:
-            warnings.warn("The use_no_gold_standard flag will be ignored because gold standard data exists")
-        elif self.use_no_gold_standard:
-            Debug.vprint("A null gold standard has been created", level=0)
-            self.gold_standard = self._create_null_prior(self.data.gene_names, self.tf_names)
+        if check_gold_standard:
+            # Create a null gold standard if the flag is set
+            if self.use_no_gold_standard and self.gold_standard is not None:
+                warnings.warn("The use_no_gold_standard flag will be ignored because gold standard data exists")
+            elif self.use_no_gold_standard:
+                Debug.vprint("A null gold standard has been created", level=0)
+                self.gold_standard = self._create_null_prior(self._gene_names, self.tf_names)
+            elif self.gold_standard is None:
+                _msg = "No gold standard found. Model scoring will be invalid. "
+                _msg += "Set worker.set_network_data_flags(use_no_gold_standard=True) to explicitly continue."
+                raise ValueError(_msg)
 
-        # Validate that some network information exists and has been loaded
-        if self.priors_data is None and self.gold_standard is None:
-            raise ValueError("No gold standard or priors have been provided")
+        if check_prior and check_gold_standard:
+            # Validate that some network information exists and has been loaded
+            if self.priors_data is None and self.gold_standard is None:
+                raise ValueError("No gold standard or priors have been provided")
 
     def input_path(self, filename):
         """
@@ -595,6 +629,10 @@ class WorkflowBaseLoader(object):
         :param tf_names: list
         :return priors: pd.DataFrame
         """
+
+        if tf_names is None:
+            raise ValueError("Unable to generate a null prior without a TF list")
+
         return pd.DataFrame(0, index=gene_names, columns=tf_names)
 
     @staticmethod
@@ -618,6 +656,24 @@ class WorkflowBaseLoader(object):
         Debug.vprint(df_name + " index: " + str(df.index[0]) + " ...", level=2)
         Debug.vprint(df_name + " columns: " + str(df.columns[0]) + " ...", level=2)
 
+    @staticmethod
+    def _check_network_labels_unique(df_name, file_name, df, raise_on_duplicate=False):
+
+        _msg = None
+
+        if not df.columns.is_unique:
+            _repeated = df.columns[df.columns.duplicated()]
+            _msg = "{name} {f}: {n} TFs are duplicated ({g})"
+            Debug.vprint(_msg.format(name=df_name, f=file_name, n=len(_repeated), g=" ".join(_repeated)), level=0)
+
+        if not df.index.is_unique:
+            _repeated = df.index[df.index.duplicated()]
+            _msg = "{name} {f}: {n} Genes are duplicated ({g})"
+            Debug.vprint(_msg.format(name=df_name, f=file_name, n=len(_repeated), g=" ".join(_repeated)), level=0)
+
+        if _msg is not None and raise_on_duplicate:
+            raise ValueError(_msg)
+
 
 class WorkflowBase(WorkflowBaseLoader):
     """
@@ -631,6 +687,7 @@ class WorkflowBase(WorkflowBaseLoader):
     # Flags to control shuffle and noise baselines
     shuffle_prior_axis = None
     make_data_noise = None
+    add_prior_noise = None
     _data_is_noise = False
 
     # The random seed for sampling, etc
@@ -690,7 +747,7 @@ class WorkflowBase(WorkflowBaseLoader):
         if not split_gold_standard_for_crossvalidation and (cv_split_axis is not None or cv_split_ratio is not None):
             warnings.warn("The split_gold_standard_for_crossvalidation flag is not set. Other options may be ignored")
 
-    def set_shuffle_parameters(self, shuffle_prior_axis=None, make_data_noise=None):
+    def set_shuffle_parameters(self, shuffle_prior_axis=None, make_data_noise=None, add_prior_noise=None):
         """
         Set parameters for shuffling labels on a prior axis. This is useful to establish a baseline.
 
@@ -700,9 +757,15 @@ class WorkflowBase(WorkflowBaseLoader):
         :param make_data_noise: Replace loaded data with simulated data that is entirely random. This retains type;
             integer data remains integer, float remains float. Gene distributions should be centered around the
             mean of gene expression in the original data, but is otherwise random.
+        :type make_data_noise: bool, None
+        :param add_prior_noise: Add random edges to the prior data. This is a numeric value between 0 and 1 such that 0
+            adds no edges, 1 sets every edge in the prior to True, 0.1 sets 10% of the edges in the prior to True, and
+            so on. Note that this will binarize the prior if it is not already binary.
+        :type add_prior_noise: numeric, None
         """
         self._set_with_warning("shuffle_prior_axis", shuffle_prior_axis)
         self._set_with_warning("make_data_noise", make_data_noise)
+        self._set_with_warning("add_prior_noise", add_prior_noise)
 
     def set_postprocessing_parameters(self, gold_standard_filter_method=None, metric=None):
         """
@@ -834,8 +897,10 @@ class WorkflowBase(WorkflowBaseLoader):
         # Filter priors to a list of regulators
         if self.tf_names is not None:
             self.priors_data = self.prior_manager.filter_to_tf_names_list(self.priors_data, self.tf_names)
-        else:
+        elif self.tf_names is None and self.priors_data is not None:
             self.tf_names = self.priors_data.columns.tolist()
+        elif self.tf_names is None:
+            raise ValueError("Either a priors_data or a tf_names file must be provided to identify regulators.")
 
         # Filter priors and expression to a list of genes
         self.filter_to_gene_list()
@@ -850,6 +915,10 @@ class WorkflowBase(WorkflowBaseLoader):
         self.priors_data, self.gold_standard = self.prior_manager.validate_priors_gold_standard(self.priors_data,
                                                                                                 self.gold_standard)
 
+        if self.add_prior_noise is not None:
+            self.priors_data = self.prior_manager.add_prior_noise(self.priors_data, self.add_prior_noise,
+                                                                  self.random_seed)
+
     def filter_to_gene_list(self):
         """
         Filter the priors and expression matrix to just genes in gene_metadata
